@@ -101,7 +101,10 @@ class InstagramExtractor(Extractor):
                         continue
 
                 url = file["display_url"]
-                yield Message.Url, url, text.nameext_from_url(url, file)
+                text.nameext_from_url(url, file)
+                if file["extension"] == "webp" and "stp=dst-jpg" in url:
+                    file["extension"] = "jpg"
+                yield Message.Url, url, file
 
     def metadata(self):
         return ()
@@ -390,10 +393,11 @@ class InstagramExtractor(Extractor):
 
     def _init_cursor(self):
         cursor = self.config("cursor", True)
-        if not cursor:
+        if cursor is True:
+            return None
+        elif not cursor:
             self._update_cursor = util.identity
-        elif isinstance(cursor, str):
-            return cursor
+        return cursor
 
     def _update_cursor(self, cursor):
         self.log.debug("Cursor: %s", cursor)
