@@ -158,11 +158,12 @@ class Job():
             raise
         except exception.GalleryDLException as exc:
             log.error("%s: %s", exc.__class__.__name__, exc)
+            log.debug("", exc_info=exc)
             self.status |= exc.code
         except OSError as exc:
             log.error("Unable to download data:  %s: %s",
                       exc.__class__.__name__, exc)
-            log.debug("", exc_info=True)
+            log.debug("", exc_info=exc)
             self.status |= 128
         except Exception as exc:
             log.error(("An unexpected error occurred: %s - %s. "
@@ -170,7 +171,7 @@ class Job():
                        "copy its output and report this issue on "
                        "https://github.com/mikf/gallery-dl/issues ."),
                       exc.__class__.__name__, exc)
-            log.debug("", exc_info=True)
+            log.debug("", exc_info=exc)
             self.status |= 1
         except BaseException:
             self.status |= 1
@@ -346,6 +347,9 @@ class DownloadJob(Job):
                 self.status |= 4
                 self.log.error("Failed to download %s",
                                pathfmt.filename or url)
+                if "error" in hooks:
+                    for callback in hooks["error"]:
+                        callback(pathfmt)
                 return
 
         if not pathfmt.temppath:
@@ -432,7 +436,8 @@ class DownloadJob(Job):
 
                     if status:
                         self.status |= status
-                        if "_fallback" in kwdict and self.fallback:
+                        if (status & 95 and   # not FormatError or OSError
+                                "_fallback" in kwdict and self.fallback):
                             fallback = kwdict["_fallback"] = \
                                 iter(kwdict["_fallback"])
                             try:
@@ -546,9 +551,15 @@ class DownloadJob(Job):
         archive_path = cfg("archive")
         if archive_path:
             archive_path = util.expand_path(archive_path)
-            archive_format = (cfg("archive-prefix", extr.category) +
-                              cfg("archive-format", extr.archive_fmt))
-            archive_pragma = (cfg("archive-pragma"))
+
+            archive_prefix = cfg("archive-prefix")
+            if archive_prefix is None:
+                archive_prefix = extr.category
+
+            archive_format = cfg("archive-format")
+            if archive_format is None:
+                archive_format = extr.archive_fmt
+
             try:
                 if "{" in archive_path:
                     archive_path = formatter.parse(
@@ -558,7 +569,10 @@ class DownloadJob(Job):
                 else:
                     archive_cls = archive.DownloadArchive
                 self.archive = archive_cls(
-                    archive_path, archive_format, archive_pragma)
+                    archive_path,
+                    archive_prefix + archive_format,
+                    cfg("archive-pragma"),
+                )
             except Exception as exc:
                 extr.log.warning(
                     "Failed to open download archive at '%s' (%s: %s)",
@@ -593,7 +607,7 @@ class DownloadJob(Job):
 
                 skip_filter = cfg("skip-filter")
                 if skip_filter:
-                    self._skipftr = util.compile_expression(skip_filter)
+                    self._skipftr = util.compile_filter(skip_filter)
                 else:
                     self._skipftr = None
         else:
@@ -617,6 +631,14 @@ class DownloadJob(Job):
             for pp_dict in postprocessors:
                 if isinstance(pp_dict, str):
                     pp_dict = pp_conf.get(pp_dict) or {"name": pp_dict}
+                elif "type" in pp_dict:
+                    pp_type = pp_dict["type"]
+                    if pp_type in pp_conf:
+                        pp = pp_conf[pp_type].copy()
+                        pp.update(pp_dict)
+                        pp_dict = pp
+                    if "name" not in pp_dict:
+                        pp_dict["name"] = pp_type
                 if pp_opts:
                     pp_dict = pp_dict.copy()
                     pp_dict.update(pp_opts)
@@ -641,7 +663,7 @@ class DownloadJob(Job):
                 except Exception as exc:
                     pp_log.error("'%s' initialization failed:  %s: %s",
                                  name, exc.__class__.__name__, exc)
-                    pp_log.debug("", exc_info=True)
+                    pp_log.debug("", exc_info=exc)
                 else:
                     pp_list.append(pp_obj)
 
@@ -655,7 +677,7 @@ class DownloadJob(Job):
         expr = options.get("filter") if options else None
 
         if expr:
-            condition = util.compile_expression(expr)
+            condition = util.compile_filter(expr)
             for hook, callback in hooks.items():
                 self.hooks[hook].append(functools.partial(
                     self._call_hook, callback, condition))
